/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SourceHelper.java 94 2006-03-26 17:11:05Z alouis $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.util;

import java.io.File;
import java.io.IOException;
import java.net.InetAddress;
import java.net.URL;
import java.net.UnknownHostException;

import org.apache.log4j.Logger;
import org.ow2.petals.tools.webadmin.cron.CollectorManager;
import org.ow2.petals.tools.webadmin.datacollector.client.CollectorClient;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.server.CollectorServer;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorClientDoesNotExistException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerNotInitializedException;

import static org.ow2.petals.jmx.api.IPetalsAdminService.EntityStates.STATE_STARTED;

/**
 * 
 * @author mcarpentier - EBM WebSourcing
 * 
 */
public final class ChannelClientHelper {

    final static Logger log = Logger.getLogger(ChannelClientHelper.class);

    public static final String PETALS_SE_CLIENT_JMX = "petals-bc-channelclient";

    private static final String SE_CLIENT_FILENAME = "jbi-archives/petals-bc-channelclient.zip";

    public static final String SE_CLIENT_STATE_NOTFOUND = "se-not-found";

    private ChannelClientHelper() {
        super();
    }

    /**
     * Try to reach petals channel client
     * 
     * @param server
     * @return
     * @throws ChannelClientHelperException
     */
    public static String getChannelClientState(String server) throws ChannelClientHelperException {
        String result = null;
        try {
            final CollectorServer collector = CollectorManager.getCollector();
            CollectorClient cc = collector.getCollectorClient(server);
            if (cc.getAdminService().getComponentByName(PETALS_SE_CLIENT_JMX) == null) {
                result = SE_CLIENT_STATE_NOTFOUND;
            } else {
                String state = cc.getAdminService().getComponentState(PETALS_SE_CLIENT_JMX);
                if (STATE_STARTED.equalsIgnoreCase(state)) {
                    if (!cc.isChannelClientReachable()) {
                        throw new ChannelClientHelperException(
                                "An internal error has occured: the PEtALS Test Client for this server is running, but it can't be reached via JMX");
                    }
                }
                return state;
            }
        } catch (DataCollectorServerException e) {
            throw new ChannelClientHelperException(e);
        } catch (DataCollectorClientDoesNotExistException e) {
            throw new ChannelClientHelperException(
                    "Can't reach PEtALS client, your server list must be out of date", e);
        } catch (DataCollectorServerNotInitializedException e) {
            throw new ChannelClientHelperException(e);
        } catch (DataCollectorClientException e) {
            throw new ChannelClientHelperException(e);
        }
        return result;
    }

    /**
     * Start the channel client installed on PEtALS
     * 
     * @param server
     * @throws ChannelClientHelperException
     */
    public static void StartChannelClient(String server) throws ChannelClientHelperException {
        try {
            final CollectorServer collector = CollectorManager.getCollector();
            collector.getCollectorClient(server).getAdminService().startComponent(
                    PETALS_SE_CLIENT_JMX);
        } catch (DataCollectorServerException e) {
            throw new ChannelClientHelperException(e);
        } catch (DataCollectorClientException e) {
            throw new ChannelClientHelperException(e);
        } catch (DataCollectorClientDoesNotExistException e) {
            throw new ChannelClientHelperException(e);
        } catch (DataCollectorServerNotInitializedException e) {
            throw new ChannelClientHelperException(e);
        }
    }

    /**
     * Install the channel client
     * 
     * @param serverName
     * @param requestURL
     * @throws ChannelClientHelperException
     */
    // TODO: make the channel client port settable, and build a page in the webconsole so users can configure the channelclient access.
    public static void InstallChannelClient(String serverName, String requestURL,String rootPath)
            throws ChannelClientHelperException {
        try {
            final CollectorServer collector = CollectorManager.getCollector();

            final int lastSeparator = requestURL.lastIndexOf("/") + 1;
            
            File f = new File(rootPath + SE_CLIENT_FILENAME);
            System.out.println(f.getAbsolutePath());
            if (!f.exists()) {
                throw new ChannelClientHelperException("ChannelClient BC File has not been installed into the webconsole : " + SE_CLIENT_FILENAME);
            }
            
            String httpUrl = requestURL.substring(0, lastSeparator) + SE_CLIENT_FILENAME;

            /**
             * try to replace possible "localhost" in the URL by the IP address *
             */
            InetAddress in = InetAddress.getLocalHost();
            String iPaddress = "";
            InetAddress[] all = InetAddress.getAllByName(in.getHostName());
            for (int i = 0; (i < all.length) && (StringHelper.isNullOrEmpty(iPaddress)); i++) {
                InetAddress ia = all[i];
                if (ia.isReachable(5000)) {
                    iPaddress = ia.getHostAddress();
                }
            }
            if (StringHelper.isNotNullAndNotEmpty(iPaddress)) {
                httpUrl = httpUrl.replace("localhost", iPaddress);
            }

            collector.getCollectorClient(serverName).getInstallService().installComponent(new URL(httpUrl));
        } catch (DataCollectorServerException e) {
            throw new ChannelClientHelperException(e);
        } catch (DataCollectorClientException e) {
            throw new ChannelClientHelperException(e);
        } catch (DataCollectorClientDoesNotExistException e) {
            throw new ChannelClientHelperException(e);
        } catch (DataCollectorServerNotInitializedException e) {
            throw new ChannelClientHelperException(e);
        } catch (UnknownHostException e) {
            throw new ChannelClientHelperException(e);
        } catch (IOException e) {
            throw new ChannelClientHelperException(e);
        }

    }
}
